"""
This file was originally sourced from the https://github.com/langchain-ai/langchain/blob/master/libs/community/langchain_community/document_loaders/github.py
and it has been modified based on the requirements provided by petercat.
"""

import base64
from typing import Callable, Optional
from github import Github
from langchain_core.documents import Document

from .repo_loader import GitFileElementType


class GithubFileLoader:
    repo_name: str
    github: Github
    file_path: str
    branch: str
    commit_id: str
    file_sha: str
    github_api_url: str = "https://api.github.com"
    token: Optional[str]
    file_size: int = 0
    file_element: GitFileElementType

    def __init__(
        self,
        file_element: GitFileElementType,
        token: Optional[str] = None,
        commit_id: Optional[str] = None,
    ):
        self.file_element = file_element
        self.repo_name = file_element.repo_name
        self.file_path = file_element.path
        self.branch = file_element.branch
        self.github = Github(self.repo_name, token)
        if not commit_id:
            self.commit_id = self._get_commit_id_by_branch(self.branch)
        else:
            self.commit_id = commit_id

    def _get_commit_id_by_branch(self, branch: str) -> str:
        repo = self.github.get_repo(self.repo_name)
        branch_info = repo.get_branch(branch)
        return branch_info.commit.sha

    def get_file_content_by_path(self, path: str) -> str:
        repo = self.github.get_repo(self.repo_name)
        file_content = (
            repo.get_contents(path, ref=self.commit_id)
            if self.commit_id
            else repo.get_contents(path)
        )
        if isinstance(file_content, list):
            print("[warn]file_content is a list")
            file_content = file_content[0]
        self.file_sha = file_content.sha
        self.file_size = file_content.size
        return base64.b64decode(file_content.content).decode("utf-8")

    def load(self) -> Document:
        content = self.get_file_content_by_path(self.file_path)
        metadata = {
            **self.file_element.model_dump(),
            "commit_id": self.commit_id,
            "file_sha": self.file_sha,
        }
        return Document(page_content=content, metadata=metadata)



from whiskerrag_types.model.knowledge import Knowledge, Task


def handle(task: Task, knowledge: Knowledge):
    # TODO: implement your code here
    print("handle task")
    print(task)
    print(knowledge)
    pass



from typing import List
from whiskerrag_types.model.knowledge import Knowledge, KnowledgeCreate, ResourceType
from whiskerrag_types.model.task import Task, TaskStatus
from whiskerrag_types.model.tenant import Tenant
from whiskerrag_utils.github.repo_loader import GitFileElementType, GithubRepoLoader
from .sha_util import calculate_sha256


async def gen_knowledge_list(
    request_body: List[KnowledgeCreate], tenant: Tenant
) -> List[Knowledge]:
    knowledge_list: List[Knowledge] = []
    for record in request_body:
        if record.knowledge_type == ResourceType.GITHUB_REPO:
            repo_knowledge_list = await get_knowledge_list_from_github_repo(
                record, tenant
            )
            knowledge_list.extend(repo_knowledge_list)
            continue
        if record.knowledge_type == ResourceType.GITHUB_FILE:
            knowledge = await get_knowledge_from_github_file(record, tenant)
            knowledge_list.append(knowledge)
            continue
        if record.knowledge_type == ResourceType.TEXT:
            knowledge = await get_knowledge_from_text(record, tenant)
            knowledge_list.append(knowledge)
            continue
        print(f"knowledge_type {record.knowledge_type} is not supported")
        continue
    return knowledge_list


async def gen_task_from_knowledge(
    knowledge_list: List[Knowledge], tenant: Tenant
) -> List[Task]:
    task_list: List[Task] = []
    for knowledge in knowledge_list:
        task = Task(
            status=TaskStatus.PENDING,
            knowledge_id=knowledge.knowledge_id,
            space_id=knowledge.tenant_id,
            tenant_id=tenant.tenant_id,
        )
        task_list.append(task)
    return task_list


async def get_knowledge_from_github_file(
    knowledge_create: KnowledgeCreate,
    tenant: Tenant,
) -> Knowledge:
    return Knowledge(
        **knowledge_create.model_dump(),
        tenant_id=tenant.tenant_id,
    )


async def get_knowledge_from_text(
    knowledge_create: KnowledgeCreate,
    tenant: Tenant,
) -> Knowledge:
    return Knowledge(
        **knowledge_create,
        file_sha256=calculate_sha256(knowledge_create.content),
        tenant_id=tenant.tenant_id,
    )


async def get_knowledge_list_from_github_repo(
    knowledge_create: KnowledgeCreate,
    tenant: Tenant,
) -> List[Knowledge]:
    repo_url = knowledge_create.source_url
    repo_name = knowledge_create.knowledge_name
    auth_info = knowledge_create.auth_info
    branch_name = None
    # Get the branch name from knowledge.source_url. If it contains tree/xxx, then xxx is the branch name. For example,
    # in https://github.com/petercat-ai/petercat/tree/fix/sqs-executes, the branch name is fix/sqs-executes.
    if "tree/" in repo_url:
        branch_name = repo_url.split("tree/")[1]
    github_loader = GithubRepoLoader(repo_name, branch_name, auth_info)
    file_list: List[GitFileElementType] = github_loader.get_file_list()
    github_repo_list: List[Knowledge] = []
    for file in file_list:
        if not file.path.endswith(".md"):
            continue
        else:
            knowledge = Knowledge(
                **knowledge_create.model_dump(
                    exclude={
                        "knowledge_type",
                        "knowledge_name",
                        "source_url",
                        "tenant_id",
                        "file_size",
                        "file_sha",
                    }
                ),
                knowledge_type=ResourceType.GITHUB_FILE,
                knowledge_name=f"{file.repo_name}/{file.path}",
                source_url=file.url,
                tenant_id=tenant.tenant_id,
                file_size=file.size,
                file_sha=file.sha,
            )
            github_repo_list.append(knowledge)
    return github_repo_list



from abc import ABC, abstractmethod
from typing import List

from whiskerrag_types.interface import (
    SettingsInterface,
    LoggerManagerInterface,
)
from whiskerrag_types.model import Knowledge, Task


class TaskEnginPluginInterface(ABC):
    settings: SettingsInterface
    logger: LoggerManagerInterface

    def __init__(
        self,
        logger: LoggerManagerInterface,
        settings: SettingsInterface,
    ):
        try:
            logger.info("TaskEngine plugin is initializing...")
            self.settings = settings
            self.logger = logger
            self.init()
            logger.info("TaskEngine plugin is initialized")
        except Exception as e:
            logger.error(f"TaskEngine plugin init error: {e}")

    @abstractmethod
    def init(self):
        pass

    @abstractmethod
    async def execute_task(self, task_id: str) -> List[Task]:
        pass

    @abstractmethod
    async def batch_execute_task(
        self, task_list: List[Task], knowledge_list: List[Knowledge]
    ) -> List[Task]:
        pass



import hashlib


def calculate_sha256(text):
    # 将文本转换为 UTF-8 编码的字节
    text_bytes = text.encode("utf-8")

    # 创建 SHA-256 哈希对象
    sha256_hash = hashlib.sha256()

    # 更新哈希对象
    sha256_hash.update(text_bytes)

    # 返回十六进制形式的哈希值
    return sha256_hash.hexdigest()



from dataclasses import Field
from datetime import datetime
from enum import Enum
from typing import Optional
from pydantic import BaseModel, field_serializer, Field


class Chunk(BaseModel):
    chunk_id: str = Field(None, description="chunk id")
    embedding: Optional[list[float]] = Field(None, description="chunk embedding")
    context: str = Field(..., description="chunk content")
    knowledge_id: str = Field(None, description="file source info")
    model_name: str = Field(..., description="model name")
    space_id: str = Field(..., description="space id")
    metadata: Optional[dict] = Field(None, description="metadata")
    created_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now().isoformat(), description="creation time"
    )
    updated_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now().isoformat(), description="update time"
    )

    @field_serializer("created_at")
    def serialize_created_at(self, created_at: Optional[datetime]):
        return created_at.isoformat() if created_at else None

    @field_serializer("updated_at")
    def serialize_updated_at(self, updated_at: Optional[datetime]):
        return updated_at.isoformat() if updated_at else None

    def update(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

        self.updated_at = datetime.now().isoformat()
        return self
